@@ -6,6 +6,28 @@
 #' * links function calls to their documentation (where possible)
 #' * in comments, translates ANSI escapes in to HTML equivalents.
 #'
+#' # Options
+#'
+#' downlit provides a number of options to control the details of the linking.
+#' They are particularly important if you want to generate "local" links.
+#'
+#' * `downlit.package`: name of the current package. Determines when
+#'   `topic_index` and `article_index`
+#'
+#' * `downlit.topic_index` and `downlit.article_index`: named character
+#'   vector that maps from topic/article name to path.
+#'
+#' * `downlit.rdname`: name of current Rd file being documented (if any);
+#'   used to avoid self-links.
+#'
+#' * `downlit.attached`: character vector of currently attached R packages.
+#'
+#' * `downlit.local_packages`: named character vector providing relative
+#'   paths (value) to packages (name) that can be reached with relative links
+#'   from the target HTML document.
+#'
+#' * `downlit.topic_path` and `downlit.article_path`: paths to reference
+#'   topics and articles/vignettes relative to the "current" file.
 #' @export
 #' @param text String of code to highlight and link.
 #' @param classes A mapping between token names and CSS class names.
@@ -13,6 +35,7 @@
 #'   that (roughly) match Pandoc and chroma (used by hugo) classes so you
 #'   can use existing themes.
 #' @param pre_class Class(es) to give output `<pre>`.
+#' @param code If `TRUE`, wrap output in a `<code>`  block
 #' @return If `text` is valid R code, an HTML `<pre>` tag. Otherwise,
 #'   `NA`.
 #' @return A string containing syntax highlighted HTML or `NA` (if `text`
@@ -23,9 +46,7 @@
 #'
 #' # Unparseable R code returns NA
 #' cat(highlight("base::t("))
-highlight <- function(text, classes = classes_chroma(), pre_class = NULL) {
-  text <- gsub("\t", "  ", text, fixed = TRUE)
-  text <- gsub("\r", "", text, fixed = TRUE)
+highlight <- function(text, classes = classes_chroma(), pre_class = NULL, code = FALSE) {
   parsed <- parse_data(text)
   if (is.null(parsed)) {
     return(NA_character_)
@@ -47,22 +68,25 @@ highlight <- function(text, classes = classes_chroma(), pre_class = NULL) {
   changed <- !is.na(out$href) | !is.na(out$class) | out$text != out$escaped
   changes <- out[changed, , drop = FALSE]
 
-  loc <- line_col(text)
+  loc <- line_col(parsed$text)
   start <- vctrs::vec_match(data.frame(line = changes$line1, col = changes$col1), loc)
   end <- vctrs::vec_match(data.frame(line = changes$line2, col = changes$col2), loc)
 
   new <- style_token(changes$escaped, changes$href, changes$class)
-  out <- replace_in_place(text, start, end, replacement = new)
+  out <- replace_in_place(parsed$text, start, end, replacement = new)
 
-  if (is.null(pre_class)) {
-    return(out)
+  if (!is.null(pre_class)) {
+    out <- paste0(
+      "<pre class='", paste0(pre_class, collapse = " "), "'>\n",
+      if (code) paste0("<code class='sourceCode R'>"),
+      out,
+      if (code) paste("</code>"),
+      "</pre>"
+    )
   }
 
-  paste0(
-    "<pre class='", paste0(pre_class, collapse = " "), "'>\n",
-    out,
-    "</pre>"
-  )
+  Encoding(out) <- "UTF-8"
+  out
 }
 
 style_token <- function(x, href = NA, class = NA) {
@@ -98,14 +122,15 @@ line_col <- function(x) {
 }
 
 parse_data <- function(text) {
+  text <- standardise_text(text)
   stopifnot(is.character(text), length(text) == 1)
 
-  expr <- safe_parse(text)
+  expr <- safe_parse(text, standardise = FALSE)
   if (is.null(expr)) {
     return(NULL)
   }
 
-  list(expr = expr, data = utils::getParseData(expr))
+  list(text = text, expr = expr, data = utils::getParseData(expr))
 }
 
 # Highlighting ------------------------------------------------------------
@@ -237,10 +262,11 @@ classes_show <- function(x, classes = classes_pandoc()) {
 
 token_href <- function(token, text) {
   href <- rep(NA, length(token))
+  to_end <- length(token) - seq_along(token) + 1
 
   # Highlight namespaced function calls. In the parsed tree, these are
   # SYMBOL_PACKAGE then NS_GET/NS_GET_INT then SYMBOL_FUNCTION_CALL/SYMBOL
-  ns_pkg <- which(token %in% "SYMBOL_PACKAGE")
+  ns_pkg <- which(token %in% "SYMBOL_PACKAGE" & to_end > 2)
   ns_fun <- ns_pkg + 2L
 
   href[ns_fun] <- map2_chr(text[ns_fun], text[ns_pkg], href_topic)
@@ -252,6 +278,9 @@ token_href <- function(token, text) {
   fun <- setdiff(fun, ns_fun)
   fun <- fun[token[fun-1] != "'$'"]
 
+  # Include custom infix operators
+  fun <- c(fun, which(token %in% "SPECIAL"))
+
   # Highlight R6 instantiation
   r6_new_call <- which(
     text == "new" & token == "SYMBOL_FUNCTION_CALL"
@@ -265,7 +294,9 @@ token_href <- function(token, text) {
 
   # Highlight packages
   lib_call <- which(
-    token == "SYMBOL_FUNCTION_CALL" & text %in% c("library", "require")
+    token == "SYMBOL_FUNCTION_CALL" &
+    text %in% c("library", "require") &
+    to_end > 3
   )
   pkg <- lib_call + 3 # expr + '(' + STR_CONST
   href[pkg] <- map_chr(gsub("['\"]", "", text[pkg]), href_package)
